'PADS PowerPCB Dynamic Drill Drawing Table D3T ver1.01
'prgm_nam:   D3T_101.bas
'codebase:   PADS PowerPCB VBscript(SAX)
'PPCB ver:   3.0x - 4.0x (ver2.x not tested)
'created:    18.06.02
'author:     Ron Olinyk, Mindlink Technologies
'decription: Generates a drill drawing table in the current PCB design using 2d lines & text,
'             removes unused drills, sorts drill symbol list
'
'Quick Run Notes
'
'Before running the program:
'
'1. A PCB design must be loaded.
'2. An up-To-Date NC Drill report must exist In the CAM subdirectory For the design.
'3. The layer where the table Is To appear Is Visible And enabled.
'
'Once above has been verified (item#2 Is very important):
'
'4. Run the program. 
'5. Adjust table parameters/values If required (mils only).
'6. Select OK.
'7. The program takes a few seconds To process Then displays the DD table.
'8. The table can be re-positioned by shift-Select / ctrl-E.
'9. Turn off old drill chart (File/CAM/drill_dwg_doc/Edit/Options/Drill Symbols/Draw Chart
'Note: only one table Is normally permitted In the design -If using PPCB v3, you must delete the 
'      table if you re-run the program otherwise load errors will result.  
'      PPCB v4 will prompt To delete.
'
'Note: the CAM Drill Symbol list is sorted and unused drills deleted (this function can be
'       disabled with the Update CAM Symbol Table option). Other than loading 2d-Line and text
'       items, no other modifications are made to the design or it's files.

Option Explicit

Dim out_txt$
Dim ch%, ht%, wd%, rw%, cl%, ox%, oy%, org_x&, org_y&, tw%, ch_wd!
Dim line_itms%, txt_itms%, lyr%, lw%, cl2%
Dim drl_dat$(), ddr%, dl2$, td_totl%, ver4 As Boolean
Dim tit As Variant, dwg_nam$, auto_imp As Boolean
Dim str_ary() As String, drl_tbl() As String
Dim cam_subdir$, cam_drlrep$, txt_ht!, lin_wd!, fcam_asc$, Asc_Hdr$
Dim TolTxtCnt%, TolTxtLen%, siz_os As Single, ld_ptr%, slt_txt%, slt_ss%
Dim Tool_ON As Boolean, Tol_ON As Boolean, Tol_Sft As Boolean, ent_err As Boolean
Dim Tcol1$, Tcol5$, Tcol6$, Ttl_Siz$, Ttl_Tol$, LstAry_Tbl$(), LAry_Sav$(), db_conv#
Dim LstAry_Tol$(), LstAry_Layr$()
Dim nn$, xx$, rr%, Dril_row$, dtbl_lyr$, XPos$, YPos$, Upd_ddst As Boolean
Const db_convmil = 38100      
Const db_convmm = 1500000     
Const db_convin = 38100 * 1000  
Const sp2 = Space$(2)
Const sp9 = Space$(9)
Const s_plus = Array(-0.5, 1, -0.5, 0, 2, -1, 0.5, 0, 0.5)                  
Const s_cros = Array(0, 0, -1, 1, 2, -1, 0, 0, 1)                           
Const s_rect = Array(-0.5, 0.5, -0.5, 1, 0, 1, 0, 0, -1, 0, -1, 1, -0.5, 1) 
Const s_diam = Array(-0.5, 0.5, -0.5, 1, 0, 0.5, -0.5, 0, -1, 0.5, -0.5, 1) 
Const s_hour = Array(0, 0, -1, 1, 0, 1, -1, 0, 0, 0)                        
Const s_bowt = Array(0, 0, -1, 1, -1, 0, 0, 1, 0, 0)                        
Const alpha_sym = Array("A", "B", "C", "D", "E", "F", "G", "H", "I", "J", "K", _
                  "L", "M", "N", "O", "P", "Q", "R", "S", "T", "U", "V", "W", _
                  "X", "Y", "Z")
Const spec_sym = Array("PLS", "CRS", "REC", "DMD", "HG", "BT")


Sub Main
    Dim a_cnt%, dd_del%, ver$
    auto_imp = True
    dwg_nam = "D3T999"                         'D3T drawing name
    If ActiveDocument.Name = "" Then
      MsgBox "A design must be loaded before running program." & vbCr _
             & "Please load a design then re-run. Aborting...", vbCritical,"Design load error"
      End
    End If  
    If CSng(Version) > 3.6 Then ver4 = True    'check if ver 4+, set flag
    If ver4 Then
      ActiveDocument.SelectObjects(,,False)
	  On Error GoTo no_dwg
	  If ActiveDocument.Drawings(dwg_nam) = dwg_nam Then
	     ActiveDocument.Drawings(dwg_nam).selected = True
	     dd_del = MsgBox ("A current Drill Dwg Table was found in the design." & vbCr _
	              & "Do you want to delete it (recommended)?" & vbCr _
	              & "-If No, you will need to manually import the new DD table." & vbCr _
	              & "or Cancel -to exit this program.", vbYesNoCancel, "DDwg Table Found")
	       If dd_del = vbYes Then
	         On Error Resume Next
	         ProcessCommand 57 '# CLI_EDIT_DELETE
	       ElseIf dd_del = vbNo Then
	         auto_imp = False
	       Else
	         End
	     End If
 	     
 	  End If
      no_dwg:
    End If
    db_conv = convdb
    ver = IIf(ver4, "V4.0", "V3.0")
    Asc_Hdr = "!PADS-POWERPCB-" & Ver & "-MILS! DESIGN DATABASE ASCII FILE 1.0"
    Call cam_asc
    Call Dril_Rep_Read 
    Call sym_table     
    For a_cnt = 0 To UBound(drl_dat,2)
       If a_cnt < 6 Then
         drl_dat(3, a_cnt) = spec_sym(a_cnt)
       Else
         drl_dat(3, a_cnt) = alpha_sym(a_cnt-6)
       End If  
    Next a_cnt
    
    If ver4 Then Slot_Data   
Call drlAry2dlg
Call Layer_Coln  
Upd_ddst = True      

	Begin Dialog UserDialog 580,357,"PowerPCB Dynamic Drill Drawing Table  1.01",.EdtListBox_Dialog ' %GRID:10,7,1,1
		OKButton 200,301,70,21
		CancelButton 120,301,60,21
		TextBox 30,28,120,21,.TBx_Siz
		TextBox 150,28,70,21,.TBx_Qty
		TextBox 220,28,60,21,.TBx_Sym
		TextBox 280,28,70,21,.TBx_Pld
		TextBox 350,28,60,21,.TBx_Tool
		TextBox 410,28,140,21,.TBx_Tol
		ListBox 30,56,520,126,LstAry_Tbl(),.LBox_Tbl
		PushButton 490,217,60,21,"&Update",.Btn_Udt
		TextBox 410,189,140,21,.TBox_Tol
		GroupBox 30,210,380,70,"",.GroupBox1
		TextBox 110,224,30,21,.Text_Wd
		TextBox 150,224,30,21,.Line_Wd
		TextBox 50,224,50,21,.Text_Ht
		CheckBox 370,11,20,14,"",.Chk_Tool
		CheckBox 430,11,20,14,"",.Chk_Tol
		Text 30,336,270,14,"Courtesy of: Mindlink Technologies",.Text5
		Text 305,298,40,14,"Layer",.Text2
		Text 30,14,280,14,"Drill Drawing Header",.Text1
		Text 110,245,70,14,"Text   Line",.Text4
		Text 60,245,40,14,"Text",.Text3
		TextBox 30,189,380,21,.TBox_Dril
		PushButton 30,301,70,21,"&Help",.Btn_Help
		ComboBox 350,294,200,49,LstAry_Layr(),.DBox_Layr
		Text 55,259,50,14,"Height",.Text6
		Text 125,259,50,14,"Width",.Text7
		PushButton 430,217,50,21,"&Fill",.Btn_Fill
		Text 154,32,60,14,"QTY",.Text8
		Text 226,32,50,14,"SYM",.Text9
		Text 286,32,60,14,"PLTD",.Text10
		TextBox 210,224,80,21,.X_Pos
		TextBox 300,224,80,21,.Y_Pos
		Text 230,245,50,14,"X Pos.",.Text12
		Text 320,245,50,14,"Y Pos.",.Text13
		Text 250,259,100,14,"Table Position",.Text14
		CheckBox 430,252,120,14,"Update CAM",.CBx_DDST
		Text 452,268,90,14,"Symbol Table",.Text11
	End Dialog
    Dim dlg As UserDialog
    Dialog dlg

    If Not Tool_ON And Tol_ON Then
      Tcol5 = Ttl_Tol
      Tcol6 = ""
      Tol_Sft = True
    ElseIf Tool_ON Then
      Tcol5 = "TOOL"
      Tcol6 = Ttl_Tol
      Tol_Sft = False
    End If  

    TolTxtLen = 0
    If Tol_ON Then
      For a_cnt = 0 To UBound(drl_dat,2)
        If Len(LstAry_Tol(a_cnt)) > 0 Then  
          TolTxtCnt = TolTxtCnt + 1                           
          If Len(LstAry_Tol(a_cnt)) > TolTxtLen Then TolTxtLen = Len(LstAry_Tol(a_cnt)) 'find longest tol string
        End If
        If Not Tool_ON And Tol_ON Then
          drl_dat(5, a_cnt) = LstAry_Tol(a_cnt)   
          Tol_Sft = True
        Else 
          drl_dat(6, a_cnt) = LstAry_Tol(a_cnt) 
        End If  
      Next a_cnt
    End If  
    Call Gen_DDTable    
    
End Sub

Sub Gen_DDTable
    Dim ver$, a_cnt%
    Dim asc_dir$, asc_file$, horz%, vert%, x1!, y1!, x2!, y2!, offs!
    Dim max_sym%, ent%, sym%, tbl_itms%, sym_itms%, line_itms%, txt_itms%
    Dim typ$, tx%, ty% , sym_dat As Variant, asc_file_ext$, c_cnt%, asc_err%, cam_err%
    
    StatusBarText = "Generating DD Table data..."
    tit = Array(Ttl_Siz, "QTY", "SYM", "PTLD", Tcol5, Tcol6)  'table header
   	
    ch_wd = 0.75 
	ht = 2       
	wd = 3       
	ox = 0       
	oy = 0       

    'Table number of rows and cols
	rw = 10      
	rw = UBound(drl_dat,2) + 2   
    
    If Tool_ON = False And Tol_ON = True Then
       cl = 5            'discount columns if Tool col off
    ElseIf Tool_ON = True And Tol_ON = False Then
       cl = 5
    ElseIf Tool_ON = False And Tol_ON = False Then
       cl = 4
    Else 
       cl = 6
    End If
    
    For a_cnt = 0 To UBound(drl_dat,2)
      For c_cnt = 1 To cl                        
        If Len(drl_dat(c_cnt, a_cnt)) > 0 Then   'only count non-blank cols
           txt_itms = txt_itms + 1
           If c_cnt = 3 And Len(drl_dat(c_cnt, a_cnt)) > 1 Then  
              txt_itms = txt_itms - 1
           End If
        End If
      Next c_cnt
    Next a_cnt
    
	tbl_itms = rw + 1 + cl + 1                           
	sym_itms = IIf(rw < 4, (rw - 1) * 2, (rw - 3) + 4)  
	sym_itms = IIf(sym_itms > 8, 8, sym_itms)            
	line_itms = tbl_itms + sym_itms + slt_ss       
	txt_itms = txt_itms + cl                      
	
    If Upd_ddst Then     
   	   Open fcam_asc & ".asc" For Append As #1
    Else
       FileCopy fcam_asc & ".asc", fcam_asc & "_CAM" & ".asc" 
       Open fcam_asc & "_CAM" & ".asc" For Append As #1       
       Print #1, vbCrLf & vbCrLf & "*END*"
       Close #1                                              
       Open fcam_asc & ".asc" For Output As #1                
    End If  
    
    'print out the ascii header
    ver = IIf(ver4, "V4.0", "V3.0")
    If Not Upd_ddst Then Print #1, "!PADS-POWERPCB-" & Ver & "-MILS! DESIGN DATABASE ASCII FILE 1.0"
    Print #1, "*LINES*      LINES ITEMS" & vbCrLf & vbCrLf & _
              dwg_nam & "        LINES    " & CStr(org_x) & "      " & CStr(org_y) & "      " & _
               CStr(line_itms) & "   " & CStr(txt_itms) 
   
    siz_os = 0
    If Len(tit(0)) > 4 Then siz_os = (Len(tit(0)) - 4) * 0.5 * ch 
                                                                   
      For a_cnt = 0 To UBound(drl_dat,2)                         
       If Len(drl_dat(1,a_cnt)) > Len(tit(0)) And Len(drl_dat(1,a_cnt)) > 4 Then 
         siz_os = (Len(drl_dat(1,a_cnt)) - 4) * 0.5 * ch  
       End If                                           
      Next a_cnt 
      
      If Len(Ttl_Tol) > TolTxtLen Then TolTxtLen = Len(Ttl_Tol)
    offs = 0
    out_txt=""
    If TolTxtLen > 4 Then offs = (TolTxtLen - 4) * 0.5 * ch
    For horz = 0 To rw           
      out_txt = out_txt & "OPEN   2   " & lw & sp2 & lyr & vbCrLf
      x1 = ox
      y1 = oy - (horz * ch * ht)
      out_txt = out_txt & CStr(x1) & "  " & CStr(y1) & vbCrLf
      x2 = ox + (cl * ch * wd) + siz_os + offs   
      y2 = oy - (horz * ch * ht)
      Print #1, out_txt & CStr(x2) & "  " & CStr(y2)
      out_txt=""
    Next horz
    offs = 0
    out_txt=""
    For vert = 0 To cl
      out_txt = out_txt & "OPEN   2   " & lw & sp2 & lyr & vbCrLf
        If vert = cl And Tol_ON And TolTxtLen > 4 Then
           offs = (TolTxtLen - 4) * 0.5 * ch   
        End If
      If vert > 0 Then offs = offs + siz_os
      x1 = ox + (vert * ch * wd) + offs
      y1 = oy
      out_txt = out_txt & CStr(x1) & "  " & CStr(y1) & vbCrLf
      x2 = ox + (vert * ch * wd) + offs
      y2 = oy - (rw * ch * ht)
      Print #1, out_txt & CStr(x2) & "  " & CStr(y2)
      out_txt=""
      offs = 0
    Next vert
    out_txt = ""
    cl2 = cl
    For sym = 0 To UBound(drl_dat,2)
      Select Case drl_dat(3, sym)
        Case spec_sym(0)
          ent = 2  
          sym_dat = s_plus
        Case spec_sym(1)
          ent = 2
          sym_dat = s_cros
        Case spec_sym(2)
          ent = 7
          sym_dat = s_rect
        Case spec_sym(3)
          ent = 6
          sym_dat = s_diam
        Case spec_sym(4)
          ent = 5
          sym_dat = s_hour
        Case spec_sym(5)
          ent = 5
          sym_dat = s_bowt
      End Select
      If Len(drl_dat(3, sym)) > 1 Then
         typ = "OPEN  " & ent & sp2 & lw & sp2 & lyr
         out_txt = out_txt & typ & vbCrLf
         Call Symbol_dwg(sym, typ, sym_dat)
      End If   
    Next sym
    offs = 0
    cl = cl2
    ty = 1
    If TolTxtLen < 4 Then TolTxtLen = 4   'min tol length box
    For tx = 1 To cl
      If (tx = 4 Or tx = 5 And Not Tol_Sft) And ty = 1 Then  
        x1 = ox + (tx * ch * wd) - (0.40 * ch) + siz_os
      ElseIf tx = 6 Or (tx = 5 And Tol_Sft) Then
        x1 = ox + (tx * ch * wd) - (0.40 * ch) + ((TolTxtLen - 4) * 0.5 * ch) + offs + siz_os
      Else
        x1 = ox + (tx * ch * wd) - (0.75 * ch) + siz_os
      End If
      y1 = oy - (ty * ch * ht) + (0.5 * ch)
      out_txt = out_txt & sp9 & CStr(x1) & sp9 & CStr(y1) & "  0.000 " & CStr(lyr) & _
                 sp9 & CStr(ch) & sp9 & tw & " N  RIGHT  DOWN" & vbCrLf
      out_txt = out_txt & tit(tx - 1)
      If tx < cl Then out_txt = out_txt & vbCrLf
    Next tx
    
    Print #1, out_txt
    out_txt = ""
    Call Tabl_Data  
    out_txt = out_txt & vbCrLf & "*END*     OF ASCII OUTPUT FILE" & vbCrLf
    Print #1, out_txt     
    Close #1               
    StatusBarText = "Sending info to PPCB..."
    
    cam_err = 1	
	If auto_imp Then    
       asc_err = ActiveDocument.ImportNetList(fcam_asc & ".asc")
    End If   
    If asc_err < 1 Or cam_err < 1 Then
       MsgBox "There was an error auto-importing the DDwg Table." & vbCr _
             & "Verify that a previous table is not in the design" & vbCr _
             & "and try manually importing the ascii file at:" & vbCr _
             & fcam_asc & ".asc", vbExclamation, "Auto-import error"
    End If
      

End Sub

Sub Symbol_dwg(sym, typ, sym_dat)
  Dim sym_x!, sym_y!, n%
   sym = sym + 1        'skip past title
   cl = 3               'sym column (3)
   sym_x = ox + (cl * ch * wd) - ch + siz_os        
   sym_y = oy - ((sym + 1) * ch * ht) + (0.5 * ch)  
   For n = 0 To UBound(sym_dat)
     If sym_dat(n) = 2 Then 
       out_txt = out_txt & typ & vbCrLf
       n = n + 1
     End If  
     out_txt = out_txt & CStr(sym_x + (sym_dat(n) * ch)) & sp2
     out_txt = out_txt & CStr(sym_y + (sym_dat(n + 1) * ch))
     n = n + 1
     If n < UBound(sym_dat) Then
       out_txt = out_txt & vbCrLf
     End If  
   Next n
   Print #1, out_txt
   out_txt = ""
   sym = sym - 1
End Sub

Sub Dril_Rep_Read
    Dim CDir$, CSDir$, CAMFullDir$, drl_lin$
    ReDim drl_dat(1 To 7,32)
    CDir = Get_CAMDir          
    CAMFullDir = CDir & cam_subdir & "\"   
    dr_lup:
    On Error GoTo dr_err
   	Open CAMFullDir & cam_drlrep For Input As #3
   	Do Until drl_lin = "Drill Sizes Report"
   	  Line Input #3, drl_lin
   	  If EOF(3) Then
   	    MsgBox "Not a drill report file (drlxxx.rep)", vbExclamation, "File Error"
   	    Exit Sub
   	  End If  
   	Loop
    ddr = -1
    Do Until EOF(3)                 
      Line Input #3, drl_lin
      If Left(drl_lin, 2) = "  " Then
        ddr = ddr + 1
        Call Str_Parse (drl_lin)     
      End If  
    Loop
    ReDim Preserve drl_dat(1 To 7,ddr)   
    Close #3
    GoTo end_err
    dr_err:
      If MsgBox("Cannot locate CAM directory or NC Drill Report." & vbCr _
               & "Confirm Drill Report has been created in:" & vbCr _
               & CAMFullDir & "   " & cam_drlrep & vbCr _
               & "and press Retry (or Cancel to Quit).", vbRetryCancel, _
               "NC Drill Report Error") = vbRetry Then GoTo dr_lup Else End
    end_err:           
End Sub

'Parse a string delimited by spaces 
Sub Str_Parse (drl_lin)
Dim ddc%, nx%, dl$, eptr%, pltd As Boolean
    For nx = 1 To Len(drl_lin)
       If Mid(drl_lin, nx, 1) <> " " Then
          eptr = InStr(nx, drl_lin, " ")- nx
          dl = Mid(drl_lin, nx, eptr)
          Call Ary_Ordr (ddc, dl)        
          ddc = ddc + 1
          nx = nx + eptr
       End If
       If Mid(drl_lin, nx - eptr, 1) = "x" Or Mid(drl_lin, nx - eptr, 1) = "-" Then 
         pltd = True
         Exit For
       End If  
    Next nx
    If pltd = True Then
      nx = Len(drl_lin)
      Do Until Mid(drl_lin, nx, 1) = " "
         dl2 = Mid(drl_lin, nx, 1)
         nx = nx - 1
      Loop
      dl = Mid(drl_lin, nx + 1, Len(drl_lin) - nx)
      Call Ary_Ordr (ddc, dl)                       
    End If
    
End Sub

Sub Tabl_Data
  Dim offs!, ty%, tx%, x1!, y1!
  offs = 0
  'offs = offs + siz_os
  For ty = 2 To UBound(drl_dat,2) + 2      'ddr + 2
    For tx = 1 To cl
      Do While drl_dat(tx,ty-2) = ""
        tx = tx + 1
        If tx > cl Then Exit For
      Loop
      If tx = 3 And Len(drl_dat(3,ty-2)) > 1 Then tx = tx + 1  
      If tx = 6 Or (tx = 5 And Tol_Sft) Then       
         x1 = ox + (tx * ch * wd) - (0.5 * ch) + ((TolTxtLen - 4) * 0.5 * ch) + offs + siz_os
      Else
         x1 = ox + (tx * ch * wd) - (0.5 * ch) + offs + siz_os  
      End If   
      y1 = oy - (ty * ch * ht) + (0.5 * ch)
      out_txt = out_txt & sp9 & CStr(x1) & sp9 & CStr(y1) & "  0.000 " & CStr(lyr) & _
                 sp9 & CStr(ch) & sp9 & tw & " N  RIGHT  DOWN" & vbCrLf
      Print #1, out_txt & drl_dat(tx,ty-2)
      out_txt = "" 
    Next tx
  Next ty
  
End Sub

Sub Ary_Ordr (ddc, dl)
Dim colx%
     Select Case ddc
       Case 0
         colx = 5       
       Case 1
         colx = 1
       Case 2
         colx = 4
         dl = IIf(dl = "x", "P", "NP")
       Case 3
         colx = 2
     End Select    
     drl_dat(colx, ddr) = dl

End Sub

Sub alpha_symbl
  Dim alfa%, x1!, y1!
  cl2 = cl
  cl = 3
    For alfa = 8 To rw 
        x1 = ox + (cl * ch * wd) - (1.25 * ch) + siz_os
        y1 = oy - (alfa * ch * ht) + (0.5 * ch)
        out_txt = out_txt & sp9 & CStr(x1) & sp9 & CStr(y1) & "  0.000 " & CStr(lyr) & _
                 sp9 & CStr(ch) & sp9 & tw & " N  RIGHT  DOWN" & vbCrLf
      Print #1, out_txt & alpha_sym(alfa-8)
      out_txt = "" 
    Next alfa
    cl = cl2
End Sub

Function Get_CAMDir
Dim PPCB$, PPCBDir$, ini_file$, data1$, camdir$, camloc$, CAM_Err As Boolean

   PPCB = UCase(FullName)         
   PPCBDir = Left(PPCB,InStr(PPCB,"POWERPCB.EXE")-1)  
   	ini_file = PPCBDir & "POWERPCB.INI"   
	Open ini_file For Input As #1         
	Do Until camloc = "CAMDir"            
	  Line Input #1,data1
	  On Error Resume Next
	  camloc = Left(data1,InStr(data1,"=")-1)
	  If EOF(1) Then 
	    MsgBox "CAM Dir not found",vbCritical,"CAM Err"
	    CAM_Err = True
	    Close #1
	    Exit Function
	  End If  
	Loop 
    Close #1
	camdir = Right(data1,Len(data1)-InStr(data1,"=")) & "\"  
	Get_CAMDir = camdir                                      
End Function

Sub cam_asc
	Dim I_file$, data1$, asc_dir$, asc_file$, NC_Drl$
	Dim strtext$, tbl_row%, tbl_col%, col%, asc_file_ext$
    ReDim str_ary(9)
    ReDim drl_tbl(3,31)
    With ActiveDocument               
       asc_dir = .path & "\"
       asc_file = Left(.Name, InStr(.Name,".")-1) ' & ".asc"
       'If Mid(asc_file, Len(asc_file)-2) <> "D3T" Then asc_file = asc_file & asc_file_ext
       .ExportASCII(asc_dir & asc_file & "_old.asc", ppcbASCIISectionCAM)
    End With
    
	I_file = asc_dir & asc_file & "_old.asc"
	Open I_file For Input As #1                        
	asc_file_ext = "_D3T"
	If Mid(asc_file, Len(asc_file)-2) <> "D3T" Then asc_file = asc_file & asc_file_ext
	fcam_asc = asc_dir & asc_file                      
	Open fcam_asc  & ".asc" For Output As #2           
	Line Input #1,data1                         
	If Mid(data1,1,14) = "!PADS-POWERPCB" Then
	   Print #2, Asc_Hdr                        
	Else
	   MsgBox "ASCII read error -incorrect format -aborting.", vbCritical, "File format error"
	   Close #1
	   Close #2
	   End
	End If
	cam_subdir = get_line("CAM_DOC_DIRECTORY", " ")    
	Do Until NC_Drl = "NC_Drill"                       
       NC_Drl = get_line("DOC_PLOT_TYPE", " ")
    Loop
    cam_drlrep = get_line("DOC_OUTPUT_FILE", " ")      
    cam_drlrep = Left(cam_drlrep, InStr(cam_drlrep,".")-1) & ".rep"
    txt_ht = CSng(get_line("CHART_TEXT_HEIGHT", " "))/db_convmil      
    lin_wd = CSng(get_line("CHART_LINE_WIDTH", " "))/db_convmil       
	Do Until data1 = "}"                               
      Line Input #1,data1                              
	  tbl_col = Str_Split(data1)
	  ReDim Preserve str_ary(tbl_col)
	  Call Trans_Array(tbl_row)
	  tbl_row = tbl_row + 1
	Loop
	ReDim Preserve drl_tbl(3,tbl_row-1)
    
	Close #1        'close old ascii file
	Kill I_file     'delete old ascii file
	
End Sub

'Function returns with right-hand (variable) section of string
Function get_line(search, charx)
Dim camloc$, data1$, n%
	Do Until camloc = search
	  Line Input #1,data1       
	  Print #2, data1           
	  On Error Resume Next
	  camloc = Left(data1,InStr(data1,charx)-1)   
	  If EOF(1) Then MsgBox "Not found":Exit Do
	Loop                                          
    get_line = Trim(Right(data1,Len(data1)-InStr(data1," ")))    
End Function

Function Str_Split(txt) 
  Dim delim%, tbl_col%, col%, dat$
   col = 1
  Do Until delim = Len(txt) + 1
    delim = InStr(col,txt," ")
    If delim = 0 Then delim = Len(txt) + 1
    If tbl_col > 0 Then 
      dat = Mid(txt, col, delim - col) 
    Else  
	  dat = Left(txt,delim -1)
	End If
	If dat <> "" Then 
	  str_ary(tbl_col) = dat
      tbl_col = tbl_col + 1
    End If  
    col = delim + 1
    'delim2 = delim + 1
  Loop	
  Str_Split = tbl_col - 1
End Function

Sub Trans_Array(tbl_row)
Dim n%
   For n = 0 To UBound(str_ary())
      drl_tbl(n,tbl_row) = str_ary(n)  'md-array drl_tbl
   Next n
End Sub

'Generate the Symbol Table for CAM ASCII section from Drill Rep array
Sub sym_table
    Dim out_row%, strtext$
    Const symbols = Array("+", "X", "Rectangle", "Diamond", "Hour_Glass", "Bow_Tie", _
                     "+A", "+B", "+C", "+D", "+E", "+F", "+G", "+H", "+I", "+J", "+K", _
                     "+L", "+M", "+N", "+O", "+P", "+Q", "+R", "+S", "+T", "+U", "+V", "+W", _
                     "+X", "+Y", "+Z")

    For out_row = 0 To UBound(drl_dat,2)
      strtext = "SYMBOL_" & out_row + 1 & " "   
	  strtext = strtext & CLng(CSng(drl_dat(1,out_row))* db_conv) & " "  'convert from basic units
  	  strtext = strtext & drl_dat(4,out_row) & " "                 
  	  strtext = strtext & symbols(out_row)                         
  	  'drl_dat(3,out_row) = symbols(out_row)
      Print #2, strtext
	Next out_row
	strtext = "}" & vbCrLf & "}"   '& vbCrLf & vbCrLf & "*END*" & vbCrLf
	
	Print #2, strtext
	Close #2             'updated CAM ascii file done

End Sub

Sub Slot_Data
    Dim npin As Object, splt$, sdril$, slen$, xpin As Object, s_ptr%, dr_ptr%, ssym$
    Dim slot_fnd As Boolean
    StatusBarText = "Searching for slots..."
    slt_txt = 0
    slt_ss = 0
    ld_ptr = UBound(drl_dat,2)              'set slot ptr beyond last drill row
    s_ptr = ld_ptr + 1
    ReDim Preserve drl_dat(1 To 7, (s_ptr + 30))
    Set npin = ActiveDocument.GetObjects(ppcbObjectTypePin,,)
    For Each xPin In npin                 'check each pin
      If xPin.SlotLength > 0 Then         
       splt = IIf(xPin.Plated,"P", "NP")  
       sdril = xPin.DrillSize      
       slen  = xpin.SlotLength      
       
       For dr_ptr = LBound(drl_dat,2) To ld_ptr      
          If drl_dat(1, dr_ptr) = sdril And drl_dat(4, dr_ptr) = splt Then
            ssym = drl_dat(3, dr_ptr)                 
            dr_ptr = ld_ptr            
            Exit For
          End If
       Next dr_ptr
       Do Until dr_ptr = s_ptr
          If drl_dat(7, dr_ptr) = slen And drl_dat(1, dr_ptr) = sdril And drl_dat(4, dr_ptr) = splt Then
             drl_dat(2, dr_ptr) = CStr(CInt(drl_dat(2, dr_ptr)) + 1)    'incr slot qty
             dr_ptr = 0
             slot_fnd = True
             Exit Do
          End If
          dr_ptr = dr_ptr + 1      
       Loop
       If dr_ptr = s_ptr And slot_fnd = False Then
          drl_dat(1, dr_ptr) = sdril
          drl_dat(2, dr_ptr) = "1"
          drl_dat(3, dr_ptr) = ssym
          drl_dat(4, dr_ptr) = splt
          drl_dat(5, dr_ptr) = " "
          drl_dat(7, dr_ptr) = slen
          s_ptr = s_ptr + 1 
          slt_txt = slt_txt + 4      
          If Len(ssym) > 1 Then 
             slt_txt = slt_txt - 1   
             slt_ss = slt_ss + IIf(ssym = "PLS" Or ssym = "CRS", 2, 1)
          End If   
       End If
       slot_fnd = False
      End If
    Next xPin
    ReDim Preserve drl_dat(1 To 7, s_ptr-1)  
    Sort_MDA_str(1)                          
    
    For dr_ptr = ld_ptr + 1 To UBound(drl_dat,2)
       drl_dat(1, dr_ptr) = drl_dat(1, dr_ptr) & " x " & drl_dat(7, dr_ptr)
    Next dr_ptr
    StatusBarText = ""
End Sub

Sub Sort_MDA_str (col)           
Dim GotIt As Boolean, temp1$, temp2$, temp3$, temp4$, temp7$, ix%  
    GotIt = True
	col = 1  
	While GotIt = True And col = 1
	GotIt = False
	  For ix = ld_ptr + 1 To (UBound(drl_dat,2)-1)
	    If drl_dat(col,ix) < drl_dat(col,ix+1) Then	
	       temp1 = drl_dat(1,ix)
	       temp2 = drl_dat(2,ix)
	       temp3 = drl_dat(3,ix)
	       temp4 = drl_dat(4,ix)
	       temp7 = drl_dat(7,ix)
		   drl_dat(1,ix) = drl_dat(1,ix+1)
		   drl_dat(2,ix) = drl_dat(2,ix+1)
		   drl_dat(3,ix) = drl_dat(3,ix+1)
		   drl_dat(4,ix) = drl_dat(4,ix+1)
		   drl_dat(7,ix) = drl_dat(7,ix+1)
		   drl_dat(1,ix+1) = temp1
		   drl_dat(2,ix+1) = temp2
		   drl_dat(3,ix+1) = temp3
		   drl_dat(4,ix+1) = temp4
		   drl_dat(7,ix+1) = temp7
		   GotIt = True
		End If
	    Next ix
	  Wend
End Sub

'D3T dialog
Function EdtListBox_Dialog(DlgItem$, Action%, SuppValue%) As Boolean
Debug.Print "Action=";Action%
Dim n_cnt%, row_sav%, tt As Variant, testx, testn
'dgstrt:
  Select Case Action%
    Case 1 ' Dialog box initialization
        DlgText "TBx_Siz", "SIZE"
        DlgText "TBx_Tool", "TOOL"   
        DlgText "TBx_Tol", "TOL"
        DlgText "Text_Ht", CStr(Txt_Ht)
        DlgText "Text_Wd", CStr(Lin_Wd)
        DlgText "Line_Wd", CStr(Lin_Wd)
        DlgValue "Chk_Tool",1
        DlgValue "Chk_Tol",1
        DlgValue "CBx_DDST",1
        DlgText "X_Pos", "1000"
        DlgText "Y_Pos", "1000"
        DlgText "TBox_Dril", DlgText("LBox_Tbl")
        DlgText "TBox_Tol", "+/-0.003"   'default value
        DlgEnable "TBox_Dril", False 
        DlgValue "DBox_Layr", 24    
        DlgEnable "TBx_Qty", False 
        DlgEnable "TBx_Sym", False 
        DlgEnable "TBx_Pld", False 
    Case 2 ' Value changing or button pressed
      Select Case DlgItem$
        Case "TBox_Dril"
             EdtListBox_Dialog = True 'do not exit the dialog 
        Case "LBox_Tbl"
             DlgText "TBox_Tol", LstAry_Tol(DlgValue("LBox_Tbl"))
             DlgText "TBox_Dril", DlgText("LBox_Tbl")
             EdtListBox_Dialog = True 
        Case "Btn_Udt"
            nn = DlgText("TBox_Tol")
            If nn = "" Then nn = " " 
            LstAry_Tbl(DlgValue("LBox_Tbl")) = LAry_Sav(DlgValue("LBox_Tbl"))
            LstAry_Tbl(DlgValue("LBox_Tbl")) = LstAry_Tbl(DlgValue("LBox_Tbl")) & nn                          
            '*Debug.Print "List2 # ";DlgValue ("LBox_Tol");" nn=";nn
            rr = DlgValue("LBox_Tbl")            
            LstAry_Tol$(rr) = nn        
            DlgListBoxArray "LBox_Tbl", LstAry_Tbl() 
            DlgValue "LBox_Tbl",rr   
            DlgText "TBox_Dril", DlgText("LBox_Tbl")
            EdtListBox_Dialog = True         'do not exit the dialog
        Case "Chk_Tool"
            If DlgValue("Chk_Tool") = 0 Then 
              DlgEnable "TBx_Tool",False
            Else
              DlgEnable "TBx_Tool",True
            End If  
            EdtListBox_Dialog = True 
        Case "Chk_Tol"
            If DlgValue("Chk_Tol") = 0 Then  
              DlgEnable "TBx_Tol",False
              DlgEnable "TBox_Tol",False
              DlgEnable "Btn_Udt",False
              DlgEnable "Btn_Fill", False
            Else
              DlgEnable "TBx_Tol",True
              DlgEnable "TBox_Tol",True
              DlgEnable "Btn_Udt",True
              DlgEnable "Btn_Fill",True
            End If
            EdtListBox_Dialog = True 
        Case "Btn_Fill"                      'fill Tol col using value in Tol testbox
            row_sav = DlgValue ("LBox_Tbl")
            For n_cnt = 0 To UBound(LstAry_Tol)
              LstAry_Tol(n_cnt) = DlgText("TBox_Tol") 
              LstAry_Tbl(n_cnt) = LAry_Sav(n_cnt) & LstAry_Tol(n_cnt) 
            Next n_cnt
            DlgListBoxArray "LBox_Tbl", LstAry_Tbl()  
            DlgValue "LBox_Tbl",row_sav      
            EdtListBox_Dialog = True       'do not exit the dialog
        Case "Btn_Help" 
            help_msg
            EdtListBox_Dialog = True  
        Case "CBx_DDST"
            Upd_ddst = IIf(DlgValue("CBx_DDST")=1, True, False)
            If DlgValue("CBx_DDST")=0 Then MsgBox "Warning: May result in Drill Dwg being out of sync.", vbExclamation,"Possible DD Sync Err"
        Case "Cancel"
            End    
      End Select
    Case 3  'Textbox or combobox text changed
        Select Case DlgItem$ 
          Case "TBox_Tol"
              nn = DlgText("TBox_Tol")  
          '*    Debug.Print "List3 # ";DlgValue("LBox_Tol");"SV ";suppvalue
          Case "Text_Ht"
             Call val_ent(tt, dlgitem$, CStr(Txt_Ht), 0, 1000)
          Case "Text_Wd"
             Call val_ent(tt, dlgitem$, CStr(Lin_Wd), 0, 50)
          Case "Line_Wd"
             Call val_ent(tt, dlgitem$, CStr(Lin_Wd), 0, 50)
          Case "X_Pos"
             Call val_ent(tt, dlgitem$, 1000, -56000, 56000)
          Case "Y_Pos"
             Call val_ent(tt, dlgitem$, 1000, -56000, 56000)
        End Select
        EdtListBox_Dialog = True 
    Case 4
         'Debug.Print "DlgFocusNN=""";DlgFocus();""""
         EdtListBox_Dialog = True 
    Case 5
  End Select
    org_x = DlgText("X_Pos")
    org_y = DlgText("Y_Pos")
    Ttl_Siz = DlgText("TBx_Siz")
    Ttl_Tol = DlgText("TBx_Tol")
    ch = DlgText("Text_Ht")
    tw = DlgText("Text_Wd")
    lw = DlgText("Line_Wd")
    lyr = DlgValue("DBox_Layr")
    Tool_ON = IIf(DlgValue("Chk_Tool") = 1,True,False) 
    Tol_ON = IIf(DlgValue("Chk_Tol") = 1,True,False) 
End Function

Sub Layer_Coln
 Dim lyr_cnt As Long, cnt%
  lyr_cnt = ActiveDocument.LayerCount
  ReDim LstAry_Layr(0 To lyr_cnt)
  For cnt = 0 To lyr_cnt
     LstAry_Layr(cnt) = ActiveDocument.LayerName(cnt)
  Next cnt
End Sub

'Dump data in drl_dat into dialog arrays
Sub drlAry2dlg
Dim a_cnt%, nsym$, sp_siz$, sp_qty$, sp_sym$, sp_pld$, sp_tool$, sp_tol$, letr%, ss1%, slen%
ReDim LstAry_Tbl(UBound(drl_dat,2))
ReDim LAry_Sav(UBound(drl_dat,2))
ReDim LstAry_Tol(UBound(drl_dat,2))
  letr = 6
  For a_cnt = 0 To UBound(drl_dat,2)
    If drl_dat(3, a_cnt) = "PLS" Then 
       nsym = "+"
    ElseIf drl_dat(3, a_cnt) = "CRS" Then 
       nsym = "X"
    Else
       nsym = drl_dat(3, a_cnt)
    End If   
    slen = IIf(Len(drl_dat(1,a_cnt))>12, Len(drl_dat(1,a_cnt)), 12)
    sp_siz = Space((slen-Len(drl_dat(1,a_cnt)))*2)
    If Len(drl_dat(1,a_cnt)) > 4 Then sp_siz = sp_siz & Space(2)
    sp_qty = Space((9-Len(drl_dat(2,a_cnt)))*2)
    sp_sym = Space((7-Len(nsym))*2)
    'If Len(nsym) = 3 Then letr = letr - 1         
    If nsym = "DMD" Or nsym = "HG" Then ss1 = -1
    sp_pld = Space(((letr-Len(drl_dat(4,a_cnt)))*2) + ss1)
    If Len(nsym) < 3 Then sp_pld = sp_pld & " "
    If drl_dat(4, a_cnt) = "P" Then sp_pld = sp_pld & " "
    letr = 6
    sp_tool = Space((8-Len(drl_dat(5,a_cnt)))*2)
    sp_tol = Space(8)
    LstAry_Tbl(a_cnt) = sp_siz & drl_dat(1, a_cnt) & sp_qty & drl_dat(2, a_cnt) & sp_sym & nsym _
                        & sp_pld & drl_dat(4, a_cnt) & sp_tool & drl_dat(5, a_cnt) & sp_tol
    LAry_Sav(a_cnt) = LstAry_Tbl(a_cnt)                        
  Next a_cnt
End Sub

Sub val_ent(tt, testx, testn, ltn&, gtn&)
             'If Val(tt) > 0 Then tt = Val(tt)
             tt = DlgText(testx)
             On Error Resume Next
             tt = Round(tt)   'round off if decimal entered
             If tt < ltn Or tt >= gtn Then
                MsgBox "Dimension must be >" & ltn & " and <=" & gtn, vbExclamation, "Entry error"
                ent_err = True
             End If
             If ent_err Then 
               DlgText testx, testn
               DlgFocus(testx)
             Else  
               DlgText testx, tt
             End If
             ent_err = False
End Sub

'Conversion factor for basic units
Function convdb
  With ActiveDocument
	If .unit = 2 Then 
	  db_conv = db_convmil 
	ElseIf .unit = 4 Then 
	  db_conv = db_convmm
	ElseIf .unit = 3 Then 
	  db_conv = db_convin 
	Else 
	  db_conv = 0
	End If 
  End With
  convdb = db_conv
End Function

Sub help_msg
MsgBox "PADS PowerPCB Dynamic Drill Drawing Table D3T ver1.01" & vbCr _
& vbCr _
& "Requirements:" & vbCr _
& "1. A PCB design must be loaded." & vbCr _
& "2. An up-to-date NC Drill report must exist in the design CAM subdir." & vbCr _
& "3. The layer where the table is to appear is visible and enabled." & vbCr _
& vbCr _
& "Usage:" & vbCr _
& "4. Run the program." & vbCr _
& "5. Adjust table parameters/values if required (values in mils only)." & vbCr _
& "6. Select OK." & vbCr _
& "7. The program processes info then displays the DD table." & vbCr _
& "8. The table can be re-positioned by shift-select / Ctrl-E." & vbCr _
& "9. Turn off old drill chart." & vbCr _
& "Note: only one table is normally permitted in the design -if using"  & vbCr _
& "PPCB v3, you must delete the table if you re-run the program." & vbCr _
& vbCr _
& "See D3T-Readme.txt for detailed instructions." & vbCr _
& vbCr _
& "The D3T program is provided free of charge courtesy of" & vbCr _
& "Mindlink Technologies. You may copy and use it as you wish." & vbCr _
& "You may also modify the script to your own needs provided" & vbCr _
& "you do not redistribute it in modified form." & vbCr _
& "Problems/comments may be directed to support@mindlinktech.com" & vbCr _
& vbCr _
& "Copyright(c) 2002 Mindlink Technologies  www.mindlinktech.com",vbInformation,"D3T Help"


End Sub
